library ieee; use ieee.std_logic_1164.all;

entity memCont is

port(
clk: 	in std_logic;
rst:	in std_logic;
brst: 	in std_logic;
rdy: 	in std_logic;
rw:	in std_logic;
bus_id:	in std_logic_vector(7 downto 0);
oe:	out std_logic;
we:	out std_logic;
addr1:	out std_logic;
addr2:	out std_logic);

end memCont;

architecture behavioral of memCont is

type state is(idle, decision, write, read1, read2, read3, read4);
signal present_state, next_state : state;

begin

seq: process(clk, rst)
begin
if (clk'event and clk = '1') then
	if (rst = '1') then
		present_state <= idle;
	else
		present_state <= next_state;
	end if;
end if;
end process seq;

comb: process(rdy, rw, bus_id, brst)
begin
case present_state is
	when idle =>
		oe <= '0';
		we <= '0';
		addr1 <= '0';
		addr2 <= '0';
		if (bus_id = "11110011") then
			if (rdy = '1') then
				next_state <= decision;
			else
				next_state <= idle;
			end if;
		else
			next_state <= idle;
		end if;

	when decision =>
		oe <= '0';
		we <= '0';
		addr1 <= '0';
		addr2 <= '0';
		if (rw = '1') then
			next_state <= read1;
		else
			next_state <= write;
		end if;

	when write =>
		we <= '1';
		oe <= '0';
		addr1 <= '0';
		addr2 <= '0';
		if (rdy = '1') then
			next_state <= idle;
		else
			next_state <= write;
		end if;
	
	when read1 =>
		oe <= '1';
		we <= '0';
		addr1 <= '0';
		addr2 <= '0';
		if (rdy = '0') then
			next_state <= read1;
		elsif (brst = '0') then
			next_state <= idle;
		else
			next_state <= read2;
		end if;

	when read2 =>
		oe <= '1';
		we <= '0';
		addr2 <= '0';
		addr1 <= '1';
		if (rdy = '1') then
			next_state <= read3;
		else
			next_state <= read2;
		end if;

	when read3 =>
		oe <= '1';
		we <= '0';
		addr1 <= '0';
		addr2 <= '1';
		if (rdy = '1') then
			next_state <= read4;
		else
			next_state <= read3;
		end if;

	when read4 =>
		oe <= '1';
		we <= '0';
		addr1 <= '1';
		addr2 <= '1';
		if (rdy = '1') then
			next_state <= idle;
		else
			next_state <= read4;
		end if;
	
	when others =>
		oe <= '0';
		we <= '0';
		addr1 <= '0';
		addr2 <= '0';
		next_state <= idle;
end case;
end process comb;
end behavioral;
